from zope import interface
import re

# mimeTypeMatch holds regular expression for matching mime-types
MTM = 'mimeTypesMatch'
# mimeType holds the preferred mime type to be returned for this
# interface
MT = "mimeType"

class ITypedFile(interface.Interface):
    """A Type of a file"""

class IBinaryFile(ITypedFile):
    """Binary file"""
IBinaryFile.setTaggedValue(MTM,re.compile('application/octet-stream'))
IBinaryFile.setTaggedValue(MT,'application/octet-stream')

class IZIPFile(IBinaryFile):
    """Zip file"""
IZIPFile.setTaggedValue(MTM,re.compile('application/x-zip'))
IZIPFile.setTaggedValue(MT,'application/x-zip')

class ITARFile(IBinaryFile):
    """Binary file"""
ITARFile.setTaggedValue(MTM,re.compile('application/x-tar'))
ITARFile.setTaggedValue(MT,'application/x-tar')

class IBZIP2File(IBinaryFile):
    """BZIP2  file"""
IBZIP2File.setTaggedValue(MTM,re.compile('application/x-bzip2'))
IBZIP2File.setTaggedValue(MT,'application/x-bzip2')

class IGZIPFile(IBinaryFile):
    """Binary file"""
IGZIPFile.setTaggedValue(MTM,re.compile('application/x-gzip'))
IGZIPFile.setTaggedValue(MT,'application/x-gzip')

class ITextFile(ITypedFile):
    """text files"""
ITextFile.setTaggedValue(MTM,re.compile('^text/.+$'))
ITextFile.setTaggedValue(MT,'text/plain')

class IImageFile(interface.Interface):
    """marker for image files"""

class IPDFFile(IBinaryFile):
    """pdf files"""
IPDFFile.setTaggedValue(MTM,re.compile('application/pdf'))
IPDFFile.setTaggedValue(MT,'application/pdf')

class IBMPFile(IImageFile, IBinaryFile):
    """jpeg file"""
IBMPFile.setTaggedValue(MTM,re.compile('image/bmp'))
IBMPFile.setTaggedValue(MT,'image/bmp')

class IJPGFile(IImageFile, IBinaryFile):
    """jpeg file"""
IJPGFile.setTaggedValue(MTM,re.compile('image/jpe?g'))
IJPGFile.setTaggedValue(MT,'image/jpeg')

class IPNGFile(IImageFile, IBinaryFile):
    """png file"""
IPNGFile.setTaggedValue(MTM,re.compile('image/png'))
IPNGFile.setTaggedValue(MT,'image/png')

class IGIFFile(IImageFile, IBinaryFile):
    """gif file"""
IGIFFile.setTaggedValue(MTM,re.compile('image/gif'))
IGIFFile.setTaggedValue(MT,'image/gif')

class IVideoFile(interface.Interface):
    """marker for video file"""

class IQuickTimeFile(IVideoFile, IBinaryFile):
    """Quicktime Video File Format"""
IQuickTimeFile.setTaggedValue(MTM,re.compile('video/quicktime'))
IQuickTimeFile.setTaggedValue(MT,'video/quicktime')

class IAVIFile(IVideoFile, IBinaryFile):
    """Quicktime Video File Format"""
IAVIFile.setTaggedValue(MTM,re.compile('video/x-msvideo'))
IAVIFile.setTaggedValue(MT,'video/x-msvideo')

class IMPEGFile(IVideoFile, IBinaryFile):
    """MPEG Video File Format"""
IMPEGFile.setTaggedValue(MTM,re.compile('video/mpe?g'))
IMPEGFile.setTaggedValue(MT,'video/mpeg')

class IMP4File(IQuickTimeFile):
    """IMP4File IPod Video File Format"""
IMP4File.setTaggedValue(MTM,re.compile('video/mp4'))
IMP4File.setTaggedValue(MT,'video/mp4')

class IFLVFile(IVideoFile, IBinaryFile):
    """Macromedia Flash FLV Video File Format"""
IFLVFile.setTaggedValue(MTM,re.compile('video/x-flv'))
IFLVFile.setTaggedValue(MT,'video/x-flv')

class IASFFile(IVideoFile, IBinaryFile):
    """Windows Media File Format"""
IASFFile.setTaggedValue(MTM,re.compile('video/x-ms-asf'))
IASFFile.setTaggedValue(MT,'video/x-ms-asf')

class IAudioFile(interface.Interface):
    """audio file"""

class IAudioMPEGFile(IAudioFile, IBinaryFile):
    """audio file"""
IAudioMPEGFile.setTaggedValue(MTM,re.compile('audio/mpeg'))
IAudioMPEGFile.setTaggedValue(MT,'audio/mpeg')

class IHTMLFile(ITextFile):
    """HTML file"""
IHTMLFile.setTaggedValue(MTM,re.compile('text/html'))
IHTMLFile.setTaggedValue(MT,'text/html')

class IXMLFile(ITextFile):
    """XML File"""
IXMLFile.setTaggedValue(MTM,re.compile('text/xml'))
IXMLFile.setTaggedValue(MT,'text/xml')

class IMSOfficeFile(IBinaryFile):
    """Microsoft Office File"""

class IMSWordFile(IMSOfficeFile):
    """Microsoft Word File"""
IMSWordFile.setTaggedValue(MTM,re.compile('application/.*msword'))
IMSWordFile.setTaggedValue(MT,'application/msword')

class IMSExcelFile(IMSOfficeFile):
    """Microsoft Excel File"""
IMSExcelFile.setTaggedValue(MTM,re.compile('application/.*excel'))
IMSExcelFile.setTaggedValue(MT,'application/msexcel')

class IMSPowerpointFile(IMSOfficeFile):
    """Microsoft Powerpoint File"""
IMSPowerpointFile.setTaggedValue(MTM,re.compile('application/.*powerpoint'))
IMSPowerpointFile.setTaggedValue(MT,'application/mspowerpoint')




from zope.lifecycleevent import ObjectModifiedEvent
from zope import interface
import interfaces

class FileTypeModifiedEvent(ObjectModifiedEvent):
    interface.implements(interfaces.IFileTypeModifiedEvent)




import zope.i18nmessageid

from zope.size.interfaces import ISized

from zope.size import byteDisplay
from interfaces import filetypes
from zope import component, interface
import os
import stat
import struct


_ = zope.i18nmessageid.MessageFactory("zope")


class ImageFileSized(object):

    interface.implements(ISized)
    
    def __init__(self, image):
        self._image = image

    @property
    def bytes(self):
        try:
            return len(self._image.data)
        except TypeError:
            data = self._image.data
            return int(os.fstat(data.fileno())[stat.ST_SIZE])
        raise NotImplementedError
    
    def sizeForSorting(self):
        '''See `ISized`'''
        return ('byte', self.bytes)

    def getImageSize(self):
        raise NotImplementedError

    def sizeForDisplay(self):
        '''See `ISized`'''

        w, h = self.getImageSize()
        if w < 0:
            w = '?'
        if h < 0:
            h = '?'
        byte_size = byteDisplay(self.bytes)
        mapping = byte_size.mapping
        if mapping is None:
            mapping = {}
        mapping.update({'width': str(w), 'height': str(h)})
        #TODO the way this message id is defined, it won't be picked up by
        # i18nextract and never show up in message catalogs
        return _(byte_size + ' ${width}x${height}', mapping=mapping)


class GIFFileSized(ImageFileSized):

    interface.implements(ISized)
    component.adapts(filetypes.IGIFFile)

    def getImageSize(self):
        data = self._image.data
        data.seek(0)
        data = data.read(24)
        size = len(data)
        width = -1
        height = -1
        if (size >= 10) and data[:6] in ('GIF87a', 'GIF89a'):
            # Check to see if content_type is correct
            w, h = struct.unpack("<HH", data[6:10])
            width = int(w)
            height = int(h)
        return width, height
    
class PNGFileSized(ImageFileSized):

    interface.implements(ISized)
    component.adapts(filetypes.IPNGFile)
    
    def getImageSize(self):
        data = self._image.data
        data.seek(0)
        data = data.read(24)
        size = len(data)
        height = -1
        width = -1
        # See PNG 2. Edition spec (http://www.w3.org/TR/PNG/)
        # Bytes 0-7 are below, 4-byte chunk length, then 'IHDR'
        # and finally the 4-byte width, height
        if ((size >= 24) and data.startswith('\211PNG\r\n\032\n')
            and (data[12:16] == 'IHDR')):
            w, h = struct.unpack(">LL", data[16:24])
            width = int(w)
            height = int(h)
        # Maybe this is for an older PNG version.
        elif (size >= 16) and data.startswith('\211PNG\r\n\032\n'):
            w, h = struct.unpack(">LL", data[8:16])
            width = int(w)
            height = int(h)
        return width, height

class JPGFileSized(ImageFileSized):

    interface.implements(ISized)
    component.adapts(filetypes.IJPGFile)
    
    def getImageSize(self):
        data = self._image.data
        data.seek(2)
        size = self.bytes
        height = -1
        width = -1
        b = data.read(1)
        try:
            w = -1
            h = -1
            while (b and ord(b) != 0xDA):
                while (ord(b) != 0xFF): b = data.read(1)
                while (ord(b) == 0xFF): b = data.read(1)
                if (ord(b) >= 0xC0 and ord(b) <= 0xC3):
                    data.read(3)
                    h, w = struct.unpack(">HH", data.read(4))
                    break
                else:
                    data.read(int(struct.unpack(">H", data.read(2))[0])-2)
                b = data.read(1)
            width = int(w)
            height = int(h)
        except struct.error:
            pass
        except ValueError:
            pass
        return width, height



from zope.lifecycleevent.interfaces import IObjectModifiedEvent
from zope import interface, schema

class IFileType(interface.Interface):

    contentType = schema.TextLine(title = u'Content Type')

class IFileTypeModifiedEvent(IObjectModifiedEvent):

    """This event is fired when the filetypes change on an object"""

class ITypeableFile(interface.Interface):

    """A file object that is typeable"""

    data = interface.Attribute('Data of the file')
    
    



from z3c.filetype import magic
import interfaces
from interfaces import filetypes
from zope.contenttype import guess_content_type
from zope import interface, component
from zope.lifecycleevent.interfaces import IObjectModifiedEvent
from zope.lifecycleevent.interfaces import IObjectCreatedEvent
from zope.event import notify
from event import FileTypeModifiedEvent

magicFile = magic.MagicFile()

def byMimeType(t):

    """returns interfaces implemented by mimeType"""

    ifaces = [iface for name, iface in vars(filetypes).items() \
              if name.startswith("I")]
    res = InterfaceSet()
    for iface in ifaces:
        mtm = iface.queryTaggedValue(filetypes.MTM)
        if mtm is not None:
            if mtm.match(t):
               res.add(iface)
    return res

def getInterfacesFor(file=None, filename=None, mimeType=None):

    """returns a sequence of interfaces that are provided by file like
    objects (file argument) with an optional filename as name or
    mimeType as mime-type
    """

    ifaces = set()
    if file is not None:
        types = magicFile.detect(file)
        for t in types:
            ifaces.update(byMimeType(t))
    if mimeType is not None:
        ifaces.update(byMimeType(mimeType))
    if filename is not None and not ifaces:
        t = guess_content_type(filename)[0]
        # dont trust this here because zope does not recognize some
        # binary files.
        if t and not t == 'text/x-unknown-content-type':
            ifaces.update(byMimeType(t))
    if not ifaces:
        ifaces.add(interfaces.filetypes.IBinaryFile)
    return InterfaceSet(*ifaces)

def applyInterfaces(obj):
    assert(interfaces.ITypeableFile.providedBy(obj))
    ifaces = InterfaceSet(*getInterfacesFor(obj.data))
    provided = set(interface.directlyProvidedBy(obj))
    for iface in provided:
        if not issubclass(iface, interfaces.filetypes.ITypedFile):
            ifaces.add(iface)
    if set(ifaces)!=provided:
        from zope.proxy import removeAllProxies
        obj = removeAllProxies(obj)
        for iface in ifaces:
            interface.directlyProvides(obj, iface)
        notify(FileTypeModifiedEvent(obj))
        return True
    return False

class InterfaceSet(object):

    """a set that only holds most specific interfaces

    >>> s = InterfaceSet(filetypes.IBinaryFile, filetypes.IImageFile)
    >>> sorted(s)
    [<InterfaceClass z3c.filetype.interfaces.filetypes.IBinaryFile>,
     <InterfaceClass z3c.filetype.interfaces.filetypes.IImageFile>]

    Now we add a jpeg file which is a subclass of all ifaces in the set
    >>> s.add(filetypes.IJPGFile)
    >>> sorted(s)
    [<InterfaceClass z3c.filetype.interfaces.filetypes.IJPGFile>]

    If we add a new which is not a subclass it is contained
    >>> s.add(filetypes.ITextFile)
    >>> sorted(s)
    [<InterfaceClass z3c.filetype.interfaces.filetypes.IJPGFile>,
     <InterfaceClass z3c.filetype.interfaces.filetypes.ITextFile>]

    """

    def __init__(self, *ifaces):
        self._data = set()
        for iface in ifaces:
            self.add(iface)
            
    def add(self, iface):
        assert(issubclass(iface, interface.Interface))
        toDelete = set()
        for i in self._data:
            if issubclass(i,iface):
                return
            if issubclass(iface, i):
                toDelete.add(i)
        self._data.add(iface)
        self._data.difference_update(toDelete)
        
    def __iter__(self):

        return iter(self._data)

@component.adapter(interfaces.ITypeableFile,IObjectModifiedEvent)
def handleModified(typeableFile, event):
    """handles modification of data"""
    if interfaces.IFileTypeModifiedEvent.providedBy(event):
        # do nothing if this is already a filetype modification event
        return
    applyInterfaces(typeableFile)

@component.adapter(interfaces.ITypeableFile,IObjectCreatedEvent)
def handleCreated(typeableFile, event):
    """handles modification of data"""
    applyInterfaces(typeableFile)



import doctest
import unittest
from zope.testing.doctestunit import DocFileSuite, DocTestSuite

def test_suite():
    
    return unittest.TestSuite(
        (
        DocFileSuite('README.txt',
                     optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,
                     ),
        DocFileSuite('magic.txt',
                     optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,
                     ),
        DocTestSuite('z3c.filetype.api',
                     optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,
                     ),
        ))

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')




from zope.cachedescriptors.property import Lazy
import interfaces
from interfaces import filetypes
from zope import interface
from zope import component

class TypedFileType(object):

    interface.implements(interfaces.IFileType)
    component.adapts(filetypes.ITypedFile)

    def __init__(self, context):
        self.context = context

    @property
    def contentType(self):
        decl = interface.Declaration(
            *interface.directlyProvidedBy(self.context))
        for iface in decl.flattened():
            if not issubclass(iface, filetypes.ITypedFile):
                continue
            mt = iface.queryTaggedValue(filetypes.MT)
            if mt is not None:
                return mt




##############################################################################
#
# Copyright (c) 2007 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Setup for z3c.filetype package

$Id: setup.py 82381 2007-12-21 10:08:32Z jukart $
"""

from setuptools import setup, find_packages

setup(
    name="z3c.filetype",
    version="1.2.0",
    namespace_packages=["z3c"],
    packages=find_packages("src"),
    package_dir={"": "src"},
    include_package_data=True,
    install_requires=[
        "setuptools",
        "zope.cachedescriptors",
        "zope.component",
        "zope.contenttype",
        "zope.event",
        "zope.i18nmessageid",
        "zope.interface",
        "zope.lifecycleevent",
        "zope.proxy",
        "zope.schema",
        "zope.size",
        ],
    extras_require={
        "test": ["zope.testing"],
        },
    zip_safe=False,
    )
